function [s,t,f,L]=MVE(signal, Nf, Nt, M, alpha, ylimit)

% Minimum Variance Estimator

% --- Input ---
% Signal: Vector containing the signal samples.
% Nf: number of frequency points
% Nt: number of time points
% M: For details see the paper cited below 
% alpha: For details see the paper cited below
% ylimit: Takes a value between 0 and 0.5 and determines the maximum 
%         frequency that is contained in s. More
%         specifically, the maximum frequency contained is Fs*ylimit, where
%         Fs is the sampling frequency.

% --- Output ---
% s: Matrix of the time frequency representation.
% t: Time instances (corresponding to the columns of s)
% F: Vector of normalized frequencies (corresponding to the rows of s)
% L: Length of the filter used. 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The script is written by Yannis Kopsinis & Elias Aboutanios for demonstration 
% purposes. Therefor, it might not be suitable in general signal cases and
% it is not optimised with respect to speed.

% For questions and/or corrections email at kopsinis@ieee.org.
% Feel free to use and modify the specific script. However, in case of
% publication the citation of the following paper will be greatly
% appreciated:

% Y. Kopsinis, E. Aboutanios, D. A. Waters, S. McLaughlin, "Instantaneous
% Frequency - based Time-Frequency representation of multi-harmonic
% signals," Journal of Acoustical Society of America (JASA),
% vol. 127(2), Feb 2010.

%Also please note that if the bat calls supplied with these functions are
%used, then the above publication MUST be referenced.


if size(signal,1)==1
    signal=signal.';
end
if alpha == 0
    L = round(1.5*M)
else
   % L = floor(M*(1+alpha)-1);
   L=2*M+alpha;
end
signal=[zeros(round(L/2),1);signal];
shift=floor((length(signal)-L)/(Nt-1));
tmp=flipud(convmtx([1:length(signal)],L));
tmp=tmp(:,L:end-L+1);
tmp=tmp(:,1:shift:end);
B=signal(tmp);
K=size(B,2);
%B = reshape(signal(1:LL*K),LL,K);
Nff=round(Nf*ylimit/0.5);
f=linspace(0,ylimit,Nff);
%LB=floor((LL-1)/alpha+1);
s = mvefun(B,f,M);
t=round(linspace(1,length(signal),K));
end

function Y = mvefun(x,f,w)

%smoothing parameter
K = size(x,1)-w+1;

S = exp(j*2*pi*(0:w-1)'*f);

partm = (1:w)'*ones(1,K) + ones(w,1)*(0:K-1);
for nn = 1:size(x,2)
    
    v = x(:,nn);
    V = v(partm);
    Rv = V*V'/K;
    Rvinv = inv(Rv);
    
    for nf = 1:length(f)
        
        s = S(:,nf);
        
        Y(nf,nn) = 1/abs(s'*Rvinv*s);
    end
end
end